package ru.ulstu.tags.service;

import org.springframework.stereotype.Service;
import ru.ulstu.tags.model.Tag;
import ru.ulstu.tags.repository.TagRepository;

import javax.transaction.Transactional;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class TagService {

    private final TagRepository tagRepository;


    public TagService(TagRepository tagRepository) {

        this.tagRepository = tagRepository;
    }

    public List<Tag> saveOrCreate(List<Tag> tags) {
        return tags
                .stream()
                .map(tag -> {
                    if (tag.getId() != null) {
                        return getExistById(tag);
                    } else {
                        Tag existTag = isExistByName(tag.getTagName());
                        return existTag != null ? existTag : create(tag);
                    }
                }).collect(Collectors.toList());
    }

    @Transactional
    public Tag getExistById(Tag tag) {
        return tagRepository.findOne(tag.getId());
    }

    @Transactional
    public Tag isExistByName(String tagName) {
        return tagRepository.findByName(tagName);
    }

    @Transactional
    public Tag create(Tag tag) {
        Tag newTag = new Tag();
        newTag.setTagName(tag.getTagName());
        newTag = tagRepository.save(newTag);
        return newTag;
    }

}
